/*
 Clock Tones with sines using PWM	9/5/2021
 GPS Clock Rev 4
 MSP430G2402 or any member with at least 2K program memory and 128 bytes RAM

 Copyright 2021
 Jed Margolin

 Permission is granted to individuals to make copies for their personal use.
 Any commercial use requires my permission.

 Port 1:
    D0  input     not used
    D1  output    Audio
    D2  input     Flag 2 (from primary MSP)
    D3  input     Flag 1 (from primary MSP)
    D4  input     not used
    D5  input     not used
    D6  input     /Test
    D7  input     Alarm

 Port 2:  D0 - D7  input   not used
*/

#include <msp430.h>

// For PWM sine wave
const unsigned int sine[32] = {
256, 293, 329, 362, 390, 414, 432, 442,
446, 442, 432, 414, 390, 362, 329, 293,
256, 219, 183, 150, 122, 98,  80,  70,
66,  70,  80,  98,  122, 150, 183, 219};

const unsigned int period1 = 1890;   // 523 Hz   C
const unsigned int period2 = 1500;   // 659 Hz   E
const unsigned int period3 = 1261;   // 784 Hz   G
const unsigned int period4 = 950;   // 1893 1044 Hz  C
const unsigned int period5 = 475;

/*
// For PWM sine wave
const unsigned int sine[16] = {
256, 329, 390, 432,
446, 432, 390, 329,
256, 183, 122, 80,
66,  80,  122, 183};

const unsigned int period1 = 3780;   // 523 Hz   C
const unsigned int period2 = 3000;   // 659 Hz   E
const unsigned int period3 = 2522;   // 784 Hz   G
const unsigned int period4 = 1900;   // 1893 1044 Hz  C
const unsigned int period5 = 950;
*/

unsigned int seconds = 1;
unsigned int period;
unsigned char start = 1;
volatile unsigned int counter = 0;   // Current location in wave array
volatile unsigned int state = 0;
unsigned char seconds_flag = 0;

unsigned char minutes_sound();
void alarm_sound(void);
void wait(unsigned int);
void waitsec(void);
void local_test1(void);
void wait10ms(void);
void local_test2(void);
void mute(void);
void unmute(void);
//**************************************************
void main(void)
{

   WDTCTL = WDTPW + WDTHOLD;                   // Stop WDT

// No crystal. Use the factory default values
   DCOCTL = CALDCO_16MHZ;      // factory calibration values
   BCSCTL1 = CALBC1_16MHZ;

// Set up the Ports
// Port 1 - I/O
   P1SEL  &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);
   P1SEL2 &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);

// Port 1 - inputs
   P1REN |= (BIT0|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);    // pullup/pulldown enabled
   P1OUT |= (BIT0|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);    //
   P1DIR &= ~(BIT0|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);   // In

// P1.1 is an output
   P1DIR |= (BIT1);     //Out
   P1OUT &= ~(BIT1);

// Port 2 - I/O
   P2SEL  &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);
   P2SEL2 &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);

// Port 2 Inputs shared with xtal (not used here)
   P2DIR &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);
   P2REN |= (BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);
   P2OUT |= (BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);

//   local_test();
   if( !(P1IN & BIT6) )      //   Do a Local Test
   {
//       local_test1();    // to get out of it remove the jumper and press Reset
//       local_test2();
       alarm_sound();
   }

   waitsec();           // wait for master MCP to get started

// Start it this way
   mute();
   seconds = 53;
   period = period1;

// Timer Interrupt for generating the PWM sine wave
   CCTL0 = CCIE;                     // CCR0 interrupt enabled
   CCR0 = sine[counter];             //  interrupts
   TACTL = TASSEL_2 | ID_0 | MC_1;   // SMCLK, contmode = count up to CCR0
   TACTL |= TACLR;          // clear timer Resets TAR, clock divider, count direction
   CCTL0 = CCIE;                     // CCR0 interrupt enabled

// Interrupt on seconds from GPS MSP on P1.3
   P1IE   |=  BIT3;   // interrupt enabled
   P1IES  &= ~BIT3;   // Low to high edge
   P1IFG  &= ~BIT3;   // Clear Interrupt flag

   __bis_SR_register(GIE);       // Interrupts without LPM0

   while(1)
   {
      if(P1IN & BIT7)
         { alarm_sound(); }
      else
         { minutes_sound();}
   }
}  // end main
//========================================
unsigned char minutes_sound()
{
   while(!seconds_flag)  // wait for seconds interrupt from GPS MSP
   {
      // During an alarm there won't be PPS
      if(P1IN & BIT7)
      { return(1); }
   }

   seconds_flag = 0;

   if( !(P1IN & BIT2) )    //  Flag 1 Seconds = 57 from GPS MSP
   {
      seconds = 57;        // sequence for end of minute
   }

   asm(" nop");    // for emulator
   switch(seconds)
   {

      case 57 :
        period = period1;
        counter = 8;
        unmute();
        wait(990);    // whole note
        mute();

        seconds = seconds + 1;
        break;

     case 58:
        period = period2;
        counter = 8;
        unmute();
        wait(990);    // whole note
        mute();

        seconds = seconds + 1;
        break;

     case 59 :
        period = period3;
        counter = 8;
        unmute();
        wait(990);    // whole note
        mute();

        seconds = seconds + 1;
        break;

     case 60 :
        period = period4;
        counter = 8;
        unmute();
        wait(50);    // whole note
        mute();

        seconds = 1;
        break;

     case 30 :
        period = period4;
        counter = 8;
        unmute();
        wait(50);    // whole note
        mute();

        seconds = seconds + 1;
        break;

     default :
        period = period1;
        counter = 8;
        unmute();
        wait(50);    // whole note
        mute();

        seconds = seconds + 1;
        // if(seconds == 60)  {seconds = 1;}
        asm(" nop");    // for emulator
        break;

   }  // end of switch

   asm(" nop");    // for emulator
   return(0);

}  // end of minutes sound
//===================================
void alarm_sound()
{
// Initialize Timer
// Set up timer Interrupts
    CCTL0 = CCIE;                     // CCR0 interrupt enabled
    CCR0 = sine[counter];             //  interrupts
    TACTL = TASSEL_2 | ID_0 | MC_1;   // SMCLK, contmode = count up to CCR0
    TACTL |= TACLR;          // clear timer Resets TAR, clock divider, count direction
    __bis_SR_register(GIE);       // Timer interrupts without LPM0

    while (1)           // main loop
    {
       period = period3;
       counter = 8;

       unmute();
       wait(500);
//---
       period = period1;
       counter = 8;

       unmute();
       wait(500);
    }
}
//=======================================
void local_test1()
{
   mute();

// Initialize Timer
// Set up timer Interrupts
    CCTL0 = CCIE;                     // CCR0 interrupt enabled
    CCR0 = sine[counter];             //  interrupts
    TACTL = TASSEL_2 | ID_0 | MC_1;   // SMCLK, contmode = count up to CCR0
    TACTL |= TACLR;          // clear timer Resets TAR, clock divider, count direction
    __bis_SR_register(GIE);       // Timer interrupts without LPM0

    seconds = 53;
    period = period1;

    while (1)           // main loop
    {
       switch(seconds)
       {

          case 57 :
            period = period1;
            counter = 8;
            unmute();
            wait(990);

            mute();
            wait(10);

            seconds = seconds + 1;
            break;

          case 58:
            period = period2;
            counter = 8;
            unmute();
            wait(990);

            mute();
            wait(10);

            seconds = seconds + 1;
            break;

          case 59 :
            period = period3;
            counter = 8;
            unmute();
            wait(990);

            mute();
            wait(10);

            seconds = seconds + 1;
            break;

          case 60 :
             period = period4;
             counter = 8;
             unmute();
             wait(50);

             mute();
             wait(950);

             seconds = 52;
             break;

          default :
            period = period1;
            counter = 8;
            unmute();
            wait(50);

            mute();
            wait(950);

            seconds = seconds + 1;
            if(seconds == 60)  {seconds = 53;}
            break;

       }  // end of switch

    }  // end of  while
}
//========================================
void local_test2()
{

// Initialize Timer
// Set up timer Interrupts
   CCTL0 = CCIE;                     // CCR0 interrupt enabled
   CCR0 = sine[counter];             //  interrupts
   TACTL = TASSEL_2 | ID_0 | MC_1;   // SMCLK, contmode = count up to CCR0
   TACTL |= TACLR;          // clear timer Resets TAR, clock divider, count direction
   __bis_SR_register(GIE);       // Timer interrupts without LPM0

   while (1)           // main loop
   {
      period = period1;
      counter = 8;

      unmute();
      wait(500);

      mute();
      wait(500);
//---
      period = period2;
      counter = 8;

      unmute();
      wait(500);

      mute();
      wait(500);
//-----------
      period = period3;
      counter = 8;

      unmute();
      wait(500);

      mute();
      wait(500);
//---------------
      period = period4;
      counter = 8;

      unmute();
      wait(500);

      mute();
      wait(500);

   }  // end of  while
}  // end of sound
//=====================================
// Timer A0 interrupt service routine
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=TIMER0_A0_VECTOR
__interrupt void Timer_A (void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(TIMER0_A0_VECTOR))) Timer_A (void)
#else
#error Compiler not supported!
#endif
{
    if(state)
    {
       P1OUT &= ~BIT1;    // Off
       // how long it will be off
       CCR0 = period - sine[counter];
       state = 0;
    }
    else
    {
        P1OUT |= BIT1;         // on
        state = 1;
        counter = counter + 1;  // increment sine counter
        if (counter == 32)      // If counter is at the end of the table
        {
           counter = 0;         // Reset counter
        }

        state = 1;
        CCR0 = sine[counter];       // how long it will be on
    }
}
//=============================================
// Port 1 interrupt service routine
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=PORT1_VECTOR
__interrupt void Port_1(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(PORT1_VECTOR))) Port_1 (void)
#else
#error Compiler not supported!
#endif
{
    seconds_flag = 1;
    P1IFG  &= ~BIT3;    //  Clear Interrupt flag
}
//=============================================
void waitsec()
{
   int n;

   for(n=200; n>0; n--)
   {wait10ms();}
}
//============================================
// wait 10 ms
void wait10ms(void)
{
   wait(41);
}
//============================================
// About 1ms per unit but not exact
void wait(unsigned int time)
{
   volatile unsigned int i,j;

   for(i=time; i>0; i--)
   {
       for(j=1550; j>0; j--);
       {
       }
   }

   return;
}
//==========================================================
// turn off timer interrupts, make P1.1 a floating input
void mute()
{
   CCTL0 = ~CCIE;     // CCR0 interrupt disabled

   P1DIR &= ~BIT1;   // In
   P1REN &= ~BIT1;    // pullup/pulldown disabled
}
//==========================================================
// make P1.1 an output again, turn timer interrupts back on
void unmute()
{
   P1DIR |= (BIT1);     //Out
   P1OUT &= ~(BIT1);

   CCTL0 = CCIE;      // CCR0 interrupt enabled
}
//==========================================================
